//+------------------------------------------------------------------+
//|                                                     CloseGui.mq4 |
//|                                                      nicholishen |
//|                         https://www.forexfactory.com/nicholishen |
//+------------------------------------------------------------------+
#property copyright "nicholishen"
#property link      "https://www.forexfactory.com/nicholishen"
#property version   "1.00"
#property strict

#define INDENT_LEFT     (8)       // left indent (including the border width)
#define INDENT_TOP      (8)       // top indent (including the border width)
#define EDIT_WIDTH      (100)     // size along the X-axis
#define EDIT_HEIGHT     (20)      // size along the Y-axis

#include <stdlib.mqh>
#include <Controls\Dialog.mqh>
#include <Controls\Button.mqh>
#include <Arrays\ArrayString.mqh>
//+------------------------------------------------------------------+
class CloseGui : public CAppDialog
{
protected:
   CButton  m_button_active;
   CButton  m_button_pending;
public:
   virtual bool      Create(  const long chart, const string name, const int subwin,
                              const int x1,const int y1,const int x2,const int y2);
                              
   virtual bool      OnEvent( const int id,const long &lparam,const double &dparam,
                              const string &sparam);
protected:
   bool              _create_buttons();
   void              _x_offset(string symbol);
   void              _x_reconcile();
   void              _x_active();
   void              _x_pending();
};
//+------------------------------------------------------------------+
EVENT_MAP_BEGIN(CloseGui)
   ON_EVENT(ON_CLICK, m_button_pending, _x_pending)
   ON_EVENT(ON_CLICK, m_button_active,  _x_active)
EVENT_MAP_END(CAppDialog)
//+------------------------------------------------------------------+
bool CloseGui::Create(const long chart,const string name,const int subwin,const int x1,const int y1,const int x2,const int y2)
{
   if(!CAppDialog::Create(chart, name, subwin, x1, y1, x2, y2)) 
      return false;
   if(!this._create_buttons())
      return false;
   return true;
}
//+------------------------------------------------------------------+
bool CloseGui::_create_buttons(void)
{
   int x1 = INDENT_LEFT;
   int y1 = INDENT_TOP;
   int x2 = x1 + EDIT_WIDTH;
   int y2 = y1 + EDIT_HEIGHT * 2;
   if(!m_button_active.Create(m_chart_id, m_name+"_close_active", m_subwin, x1, y1, x2, y2))
      return false;
   m_button_active.Text("Close Active");
   x1 = x2 + INDENT_LEFT;
   x2 = x1 + EDIT_WIDTH;
   if(!m_button_pending.Create(m_chart_id, m_name+"_close_pending", m_subwin, x1, y1, x2, y2))
      return false;
   m_button_pending.Text("Close Pending"); 
   m_button_pending.ColorBackground(clrRed);
   m_button_active.ColorBackground(clrOrange);
   if(!this.Add(m_button_pending) || !this.Add(m_button_active))                                                            
      return(false);
   return true;
}
//+------------------------------------------------------------------+
void CloseGui::_x_active(void)
{
   CArrayString symbols;
   for(int i=OrdersTotal()-1; i>=0; i--)
      if(OrderSelect(i, SELECT_BY_POS) && OrderType() < 2)
         if(symbols.SearchLinear(OrderSymbol()) < 0)
            symbols.Add(OrderSymbol());
   for(int i=symbols.Total()-1; i>=0; i--)
      this._x_offset(symbols[i]);
   this._x_reconcile();
}
//+------------------------------------------------------------------+
void CloseGui::_x_reconcile(void)
{
   for(int i=OrdersTotal()-1; i>0 && !IsStopped(); i--){
      if(OrderSelect(i, SELECT_BY_POS) && OrderType() < 2){
         int type = OrderType();
         int ticket = OrderTicket();
         string symbol = OrderSymbol();
         for(int j=i-1; OrderSelect(j, SELECT_BY_POS); j--){
            if(OrderSymbol() == symbol && OrderType() < 2 && OrderType() != type){
               if(!OrderCloseBy(OrderTicket(),ticket))
                  Print("OrderCLoseByError: ",ErrorDescription(_LastError));
               i = OrdersTotal();
               break; 
            }
         }
      }
   }   
}
//+------------------------------------------------------------------+
void CloseGui::_x_offset(string symbol)
{
   double net_position = 0.0;
   for(int i=OrdersTotal()-1; i>=0; i--){
      if(OrderSelect(i, SELECT_BY_POS) && OrderSymbol() == Symbol() && OrderType() < 2 ){
         if(OrderType() == OP_BUY)
            net_position += OrderLots();
         else
            net_position -= OrderLots();
      }
   }
   net_position = NormalizeDouble(net_position, 2);
   if(net_position > 0.0){
      if(OrderSend(symbol, OP_SELL, fabs(net_position), SymbolInfoDouble(symbol, SYMBOL_BID), 10, 0., 0.) < 0)
         Alert(ErrorDescription(_LastError));
   }
   else if(net_position < 0.0){
      if(OrderSend(symbol, OP_BUY, fabs(net_position), SymbolInfoDouble(symbol, SYMBOL_ASK), 10, 0., 0.) < 0)
         Alert(ErrorDescription(_LastError));
   }
}
//+------------------------------------------------------------------+
void CloseGui::_x_pending(void)
{
   for(int i=OrdersTotal()-1; i>=0; --i)
      if(OrderSelect(i, SELECT_BY_POS) && OrderType() > 1)
         if(!OrderDelete(OrderTicket()))
            Alert(ErrorDescription(_LastError));
}
//+------------------------------------------------------------------+
CloseGui gui;

int OnInit()
{
   if(!gui.Create(0, "Close-All", 0, 100, 100, 335, 190)
      || !gui.Run()
   ){
      return INIT_FAILED;
   }
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   gui.Destroy();
}
//+------------------------------------------------------------------+
void OnTick(){}
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
{
   gui.ChartEvent(id, lparam, dparam, sparam);
}
//+------------------------------------------------------------------+
